#!/bin/bash

# Vérifier si ImageMagick est installé
if ! command -v convert &> /dev/null; then
    echo "ImageMagick n'est pas installé. Veuillez l'installer."
    exit 1
fi

# Vérifier si l'argument de l'image est fourni
if [ $# -eq 0 ]; then
    echo "Usage: $0 <image_source>"
    exit 1
fi

# Récupérer le nom du fichier source
image_source="$1"

# Vérifier si le fichier source existe
if [ ! -f "$image_source" ]; then
    echo "Le fichier source '$image_source' n'existe pas."
    exit 1
fi

# Utiliser la commande "file" pour vérifier le type de fichier
if ! file "$image_source" | grep -q "image"; then
    echo "Le fichier '$image_source' n'a pas l'air d'être une image. Je tente le coup quand même."
fi

# Enlever l'extension du nom de fichier
filename_no_extension="${image_source%.*}"

# Redimensionner l'image en 1400x1400 pixels et la convertir en JPG
convert "$image_source" -resize 1400x1400 "${filename_no_extension}.jpg"

echo "L'image a été redimensionnée en 1400x1400 pixels et enregistrée en tant que '${filename_no_extension}.jpg'."
